/* 
   [sd.c]
   (c) Simon Kueppers
   20.12.06 Germany

   This file is the interface to the sdcard. For this case, it emulates a ram drive with a given size.
   (DISK_SIZE). 
   

   PS:
   I tried to comment the code as best as i was able to. This is a little bit confusing
   for a german, so please excuse if something is not that right it should be.
   Have fun with this code but remember the BEER LICENSE when you copy+paste something
   from here :-)

*/

#include <stdio.h>									//fopen() fclose() fread()
#include "stdint.h"									//Vartypedefs
#include "sd.h"										//function prototypes for sd card access

/*
   Defines for Ramdisk
*/
#define		DISK_SIZE		8192UL
#define		BLK_SIZE		512 
#define		PAGE_COUNT		(DISK_SIZE/BLK_SIZE)

uint8_t disk[DISK_SIZE];							//declare disk with given size



/*
   Inits SD-Card interface.
   Should be run once at startup
*/
void sd_init(void)
{
	uint32_t i;
	uint8_t *ptemp = (uint8_t *)&disk;				//set a pointer to start of disk

	FILE* filehandle = fopen("img.img", "rb");		//open pre-made diskimage
	for(i=0; i<DISK_SIZE; i++)
		fread(ptemp++, 1, 1, filehandle);			//and write it to ramdisk bytewise
	fclose(filehandle);

}

/*
   Writes nBytes bytes of data to disk at pageadress
*/
void sd_write(uint32_t adress, void *p_src)
{
	uint16_t nBytes = BLK_SIZE;
	uint8_t *p_dst = ((uint8_t *) disk) + adress;	//set a pointer to desired position on disk

	while (nBytes--)								//copy
		*p_dst++ =  *((uint8_t *) p_src)++;			//bytewise from source to destination
	
}

/*
   Reads 512 bytes of data from disk at pageadress
*/
void sd_read (uint32_t adress, void *p_dst)
{
	uint16_t nBytes = BLK_SIZE;
	uint8_t *p_src = ((uint8_t *) disk) + adress;	//set a pointer to desired position on disk

	while (nBytes--)								//copy
		*((uint8_t *) p_dst)++ = *p_src++;			//bytewise from source to destination 

}

/*
   Reads nBytes bytes of data from disk at pageadress
*/
void sd_nread (uint32_t adress, void *p_dst, uint16_t nBytes)
{
	uint8_t *p_src = ((uint8_t *) disk) + adress;	//set a pointer to desired position on disk

	while (nBytes--)								//copy bytewise from source to destination
		*((uint8_t *) p_dst)++ = *p_src++;			//until there is nothing to read anymore

}

/*
   Writes nBytes bytes of data to disk at pageadress and pads enough zeros to write 512byte
*/
void sd_nwrite (uint32_t adress, void *p_src, uint16_t nBytes)
{
	uint16_t nPadbytes = BLK_SIZE - nBytes;
	uint8_t *p_dst = ((uint8_t *) disk) + adress;	//set a pointer to desired position on disk

	while (nBytes--)								//copy bytewise from source to destination
		*p_dst++ =  *((uint8_t *) p_src)++;			//until there is nothin to write anymore

	while (nPadbytes--)								//pad remaining page size
		*p_dst++ = 0;								//with zeros ('0')
	
}