/* 
   [time.c]
   (c) Simon Kueppers
   20.12.06 Germany

   This file holds functions to get/set or convert time/timestamps. time.c holds a 
   static variable (a timestamp) of the current time. It is the time in second 
   since 1.1.1970. This format equals to the NTP file format and is also used
   by the ufs filesystem. To convert the timestamp into a more-readable format,
   use the "convtime()" function. it converts the timestamp given as a parameter 
   to a t_time struct (defined in time.h)
   

   PS:
   I tried to comment the code as best as i was able to. This is a little bit confusing
   for a german, so please excuse if something is not that right it should be.
   Have fun with this code but remember the BEER LICENSE when you copy+paste something
   from here :-)

*/

#include "time.h"									//function prototypes and defines
#include "stdint.h"									//Vartypedefs


static volatile uint32_t currenttime = 0xAAAAAAAA;

uint8_t DaysInMonth[12] =							//array with information how many days a specific month has
{
	31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
};


/*
   This function should be called every second to get a stable 
   RTC base. (Use a timer or something like that)
*/
void rtc_irq(void)
{ 
	currenttime++;
}

/*
   Use this function to convert the a timestamp (for example
   the currenttime) to a more readable struct t_time.
   This function is copy+pasted from peter danneggers code.
   Its a litte bit modified, but that should not mind.
   I have not commented this code snippet, because I do not
   know myself what this code does exactly. The main point is
   that it does function ;) (as far as i tested the function)
*/
void convtime(uint32_t sec, struct t_time* time)
{
	uint16_t DaysInYear;
	uint8_t leap;
	uint16_t year;
	uint8_t month = 0;
	uint16_t day;
	uint8_t hour;
	uint8_t minute;
	uint8_t second;
	uint8_t weekday;

	second = (uint8_t) sec % 60;
	sec /= 60;
	minute = (uint8_t) sec % 60;
	sec /= 60;
	hour = (uint8_t) sec % 24;
	sec /= 24;

	day = (uint8_t) sec;
	weekday = (day + 6) % 7;
	leap = 4 - ((1900 - 1) / 100 & 3);
	year = 0;

	while(1)
	{
		DaysInYear = 365;
		if((year & 3) == 0)
		{
			DaysInYear = 366;
			if((year == 0) || (year == 100) || (year == 200))
			{
				leap--;
				if(leap)
					DaysInYear = 365;
			}
		}
		
		if(day < DaysInYear)
			break;
		
		day -= DaysInYear;
		year++;
	}
	
	year += 1900;
	
	if((DaysInYear & 1) && (day > 58))
		day++;
	
	while(day >= DaysInMonth[month])
		day -= DaysInMonth[month++];

	month += 1;
	day += 1;

	
	if(!((month < 3) || (month > 10)))
	{
		if((day - weekday >= 25) && (weekday || hour >= 2))
		{
			if(month != 10)
				hour++;
		}else
		{
			if(month != 3)
				hour++;
		}

	}
	
	if(hour == 24)
	{
		hour = 0;
		
		if(day == DaysInMonth[month-1])
		{
			day = 0;
			month += 1;
		}
		day++;
	}
		
	time->year = year;
	time->month = month;
	time->day = (uint8_t) day;
	time->hour = hour;
	time->minute = minute;
	time->second = second;

}

/*
   This function lets you get the current time from everywhere else
*/
uint32_t gettime(void)
{
	uint32_t time;
	//cli();										//If you do not use an 32 bit system, be sure to make this
	time = currenttime;								//access atomic! 
	//sei();										//For AVRs you might use cli() and sei()

	return time;
}

/*
   This function lets you set the current time from everywhere else
*/
void settime(uint32_t seconds)
{
	//cli();										//same thing here
	currenttime = seconds + GMT;
	//sei();
}

